@@ -1306,7 +1306,16 @@ pub enum ExternalMsg {
     /// Quit with returncode zero (success).
     Quit,
 
-    /// Print selected paths if it's not empty, else, print the focused node's path.
+    /// Print $PWD and quit.
+    PrintPwdAndQuit,
+
+    /// Print the path under focus and quit. It can be empty string if there's nothing to focus.
+    PrintFocusPathAndQuit,
+
+    /// Print the selected paths and quit. It can be empty is no path is selected.
+    PrintSelectionAndQuit,
+
+    /// Print the selected paths if it's not empty, else, print the focused node's path.
     PrintResultAndQuit,
 
     /// Print the state of application in YAML format. Helpful for debugging or generating
@@ -1363,8 +1372,6 @@ pub enum MsgOut {
     Refresh,
     ClearScreen,
     Quit,
-    PrintResultAndQuit,
-    PrintAppStateAndQuit,
     Debug(String),
     Call(Command),
     CallSilently(Command),
@@ -1377,6 +1384,11 @@ pub enum MsgOut {
     StartFifo(String),
     StopFifo,
     ToggleFifo(String),
+    PrintPwdAndQuit,
+    PrintFocusPathAndQuit,
+    PrintSelectionAndQuit,
+    PrintResultAndQuit,
+    PrintAppStateAndQuit,
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
@@ -1744,6 +1756,9 @@ impl App {
                 ExternalMsg::LogWarning(l) => self.log_warning(l),
                 ExternalMsg::LogError(l) => self.log_error(l),
                 ExternalMsg::Quit => self.quit(),
+                ExternalMsg::PrintPwdAndQuit => self.print_pwd_and_quit(),
+                ExternalMsg::PrintFocusPathAndQuit => self.print_focus_path_and_quit(),
+                ExternalMsg::PrintSelectionAndQuit => self.print_selection_and_quit(),
                 ExternalMsg::PrintResultAndQuit => self.print_result_and_quit(),
                 ExternalMsg::PrintAppStateAndQuit => self.print_app_state_and_quit(),
                 ExternalMsg::Debug(path) => self.debug(path),
@@ -1864,7 +1879,11 @@ impl App {
 
     fn focus_previous(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
-            dir.focus = dir.focus.max(1) - 1;
+            dir.focus = if dir.focus == 0 {
+                dir.total.max(1) - 1
+            } else {
+                dir.focus.max(1) - 1
+            };
         };
         Ok(self)
     }
@@ -1894,7 +1913,11 @@ impl App {
 
     fn focus_next(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
-            dir.focus = (dir.focus + 1).min(dir.total.max(1) - 1);
+            dir.focus = if (dir.focus + 1) == dir.total {
+                0
+            } else {
+                dir.focus + 1
+            }
         };
         Ok(self)
     }
@@ -2103,7 +2126,10 @@ impl App {
     }
 
     pub fn focus_path(self, path: &str, save_history: bool) -> Result<Self> {
-        let pathbuf = PathBuf::from(path);
+        let mut pathbuf = PathBuf::from(path);
+        if pathbuf.is_relative() {
+            pathbuf = PathBuf::from(self.pwd()).join(pathbuf);
+        }
         if let Some(parent) = pathbuf.parent() {
             if let Some(filename) = pathbuf.file_name() {
                 self.change_directory(&parent.to_string_lossy().to_string(), false)?
@@ -2264,7 +2290,10 @@ impl App {
     }
 
     fn select_path(mut self, path: String) -> Result<Self> {
-        let path = PathBuf::from(path);
+        let mut path = PathBuf::from(path);
+        if path.is_relative() {
+            path = PathBuf::from(self.pwd()).join(path);
+        }
         let parent = path.parent().map(|p| p.to_string_lossy().to_string());
         let filename = path.file_name().map(|p| p.to_string_lossy().to_string());
         if let (Some(p), Some(n)) = (parent, filename) {
@@ -2517,6 +2546,21 @@ impl App {
         Ok(self)
     }
 
+    fn print_pwd_and_quit(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::PrintPwdAndQuit);
+        Ok(self)
+    }
+
+    fn print_focus_path_and_quit(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::PrintFocusPathAndQuit);
+        Ok(self)
+    }
+
+    fn print_selection_and_quit(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::PrintSelectionAndQuit);
+        Ok(self)
+    }
+
     fn print_result_and_quit(mut self) -> Result<Self> {
         self.msg_out.push_back(MsgOut::PrintResultAndQuit);
         Ok(self)
